@@ -1461,6 +1461,9 @@ get_id3v23_tags (id3v24frame           frame,
 		case ID3V24_TOLY:
 			tag->toly = word;
 			break;
+		case ID3V24_TCOM:
+			tag->composer = word;
+			break;
 		case ID3V24_TIT1:
 			tag->title1 = word;
 			break;
@@ -2180,6 +2183,9 @@ extract_mp3 (const gchar          *uri,
 		tracker_sparql_builder_subject_iri (preupdate, md.album_uri);
 		tracker_sparql_builder_predicate (preupdate, "a");
 		tracker_sparql_builder_object (preupdate, "nmm:MusicAlbum");
+		/* FIXME: nmm:albumTitle is now deprecated
+		 * tracker_sparql_builder_predicate (preupdate, "nie:title");
+		 */
 		tracker_sparql_builder_predicate (preupdate, "nmm:albumTitle");
 		tracker_sparql_builder_object_unvalidated (preupdate, md.album);
 
@@ -2295,6 +2301,7 @@ extract_mp3 (const gchar          *uri,
 	                          md.album,
 	                          filename);
 	g_free (md.albumart_data);
+	g_free (md.albumart_mime);
 
 	id3v2tag_free (&md.id3v22);
 	id3v2tag_free (&md.id3v23);
