@@ -41,6 +41,7 @@
 #include <libtracker-common/tracker-ioprio.h>
 #include <libtracker-common/tracker-log.h>
 #include <libtracker-common/tracker-ontologies.h>
+#include <libtracker-common/tracker-locale.h>
 
 #include <libtracker-data/tracker-data-manager.h>
 #include <libtracker-data/tracker-data-backup.h>
@@ -56,6 +57,7 @@
 #include "tracker-backup.h"
 #include "tracker-store.h"
 #include "tracker-statistics.h"
+#include "tracker-locale-change.h"
 
 #define ABOUT	  \
 	"Tracker " PACKAGE_VERSION "\n"
@@ -502,7 +504,7 @@ main (gint argc, gchar *argv[])
 	                                update_cache_size,
 	                                busy_callback,
 	                                busy_user_data,
-	                                "Journal replaying")) {
+	                                "Initializing")) {
 
 		g_object_unref (db_config);
 		g_object_unref (notifier);
@@ -516,12 +518,15 @@ main (gint argc, gchar *argv[])
 		goto shutdown;
 	}
 
+	/* Setup subscription to get notified of locale changes */
+	tracker_locale_change_initialize_subscription ();
+
 	tracker_dbus_register_prepare_class_signal ();
 
 	tracker_events_init (get_notifiable_classes);
 	tracker_writeback_init (get_writeback_predicates);
 
-	tracker_store_set_active (TRUE);
+	tracker_store_set_active (TRUE, NULL, NULL);
 
 	g_message ("Waiting for D-Bus requests...");
 
@@ -552,6 +557,8 @@ main (gint argc, gchar *argv[])
 	tracker_writeback_shutdown ();
 	tracker_events_shutdown ();
 
+	tracker_locale_change_shutdown_subscription ();
+
 	tracker_dbus_shutdown ();
 	tracker_data_manager_shutdown ();
 	tracker_log_shutdown ();
